package ca.canadapost.cpcdp.returns;

import ca.canadapost.cpcdp.returns.stub.openreturn.*;
import ca.canadapost.cpcdp.returns.stub.openreturn.DomesticAddressDetailsType;
import ca.canadapost.cpcdp.returns.stub.openreturn.Messages;
import ca.canadapost.cpcdp.returns.stub.openreturn.PrintPreferencesType;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import org.apache.cxf.interceptor.LoggingInInterceptor;
import org.apache.cxf.interceptor.LoggingOutInterceptor;
import ca.canadapost.cpcdp.returns.util.ClientUtil;

import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPFaultException;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Map;

/**
 * Sample code for the CreateOpenReturnTemplate Canada Post service.
 * <p>
 * The CreateOpenReturnTemplate service is used to request creation of generic
 * labels for retrieval and printing. The sender address and parcel weight are
 * unknown. Each label has a unique barcode, but the rest of the data is the same.
 * The labels can be distributed as part of the original shipment, or sent to a
 * specific individual.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment.
 * Use your development key username and password for the web service credentials.
 */
public class CreateOpenReturnTemplateClient {
	// Main class for local testing only
    public static void main(String[] args) {
        try{
            // Your username, password and customer number are imported from the following file: CPCWS_SOAP_Returns_Java_Samples/user.properties
            Map<String, String> userProperties = ClientUtil.getUserProperties("user.properties");
            String username = userProperties.get("username");
            String password = userProperties.get("password");
            String mailedBy = userProperties.get("mailedBy");

            OpenReturnService service = new OpenReturnService();
            OpenReturnPortType port = service.getOpenReturnPort();

            //set endpoint
            Map ctx = ((BindingProvider) port).getRequestContext();
            ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/rs/soap/openreturn/v2");

            //wss4j interceptor -  add UsernameToken to soap header
            Client client = ClientProxy.getClient(port);
            client.getOutInterceptors().add(ClientUtil.getUsernameTokenInterceptor(username, password));

            //create request
            DomesticAddressDetailsType domesticAddressDetailsType = new DomesticAddressDetailsType();
            domesticAddressDetailsType.setAddressLine1("123 Postal Drive");
            domesticAddressDetailsType.setCity("Ottawa");
            domesticAddressDetailsType.setProvince("ON");
            domesticAddressDetailsType.setPostalCode("K1P5Z9");

            DomesticAddressType domesticAddressType = new DomesticAddressType();
            domesticAddressType.setName("John Doe");
            domesticAddressType.setCompany("ACME Corp.");
            domesticAddressType.setDomesticAddress(domesticAddressDetailsType);

            PrintPreferencesType printPreferencesType = new PrintPreferencesType();
            printPreferencesType.setOutputFormat("8.5x11");
            printPreferencesType.setEncoding("PDF");

            SettlementInfoType settlementInfoType = new SettlementInfoType();
            settlementInfoType.setContractId("0040662505");

            OpenReturnType openReturnType = new OpenReturnType();
            openReturnType.setMaxNumberOfArtifacts(new BigInteger("10"));
            openReturnType.setServiceCode("DOM.EP");
            openReturnType.setReceiver(domesticAddressType);
            openReturnType.setPrintPreferences(printPreferencesType);
            openReturnType.setSettlementInfo(settlementInfoType);

            CreateOpenReturnTemplateRequest request = new CreateOpenReturnTemplateRequest();
            request.setLocale("FR");
            request.setMailedBy(mailedBy);
            request.setOpenReturn(openReturnType);

            //execute response
            CreateOpenReturnTemplateResponse response = port.createOpenReturnTemplate(request);

            // Retrieve values from response object
            if(response.getMessages() == null){
                System.out.println("Artifacts Remaining: " + response.getOpenReturnInfo().getArtifactsRemaining());
                System.out.println("Template Id: " + response.getOpenReturnInfo().getTemplateId());
            } else {
                // Assume Error
                for(Messages.Message m : response.getMessages().getMessage()){
                    System.out.println("Error Code: " + m.getCode());
                    System.out.println("Error Message: " + m.getDescription());
                }
            }
        }
        catch (SOAPFaultException e) {
            System.out.println("Fault Code: " + e.getFault().getFaultCode());
            System.out.println("Fault Reason: " + e.getFault().getFaultString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
